<?php

namespace Drupal\interactive_globe\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;

/**
 * Provides an Interactive Globe Block.
 *
 * @Block(
 *   id = "interactive_globe_block",
 *   admin_label = @Translation("Interactive Globe Block"),
 *   category = @Translation("Custom")
 * )
 */
class InteractiveGlobeBlock extends BlockBase
{

  /**
   * {@inheritdoc}
   */
    public function blockForm($form, FormStateInterface $form_state): array
    {
        $form = parent::blockForm($form, $form_state);

        $form['globe_background'] = [
        '#type' => 'managed_file',
        '#title' => $this->t('Globe Background Image'),
        '#upload_location' => 'public://globe_background/',
        '#default_value' => isset($this->configuration['globe_background']) ?
          [$this->configuration['globe_background']] : [],
        '#required' => true,
        ];

      // Fields for each icon
        for ($i = 1; $i <= 6; $i++) {
            $form["icon_$i"] = [
            '#type' => 'managed_file',
            '#title' => $this->t("Icon $i Image"),
            '#upload_location' => 'public://globe_icons/',
            '#default_value' => isset($this->configuration["icon_$i"]) ? [$this->configuration["icon_$i"]] : [],
            '#required' => true,
            ];
            $form["text_$i"] = [
            '#type' => 'text_format',
            '#title' => $this->t("Text for Icon $i"),
            '#default_value' => $this->configuration["text_$i"]['value'] ?? '',
            '#format' => $this->configuration["text_$i"]['format'] ?? 'full_html',
            '#allowed_formats' => ['full_html', 'basic_html', 'restricted_html'],
            ];
        }

        return $form;
    }

  /**
   * {@inheritdoc}
   * @throws EntityStorageException
   */
    public function blockSubmit($form, FormStateInterface $form_state): void
    {
        parent::blockSubmit($form, $form_state);

      // Save the file ID for the background image
        $bg_fid = $form_state->getValue(['globe_background', 0]);
        if ($bg_fid) {
            $this->configuration['globe_background'] = $bg_fid;
            $file = File::load($bg_fid);
            if ($file) {
                $file->setPermanent();
                $file->save();
            }
        }

      // Save file IDs for the icons
        for ($i = 1; $i <= 6; $i++) {
            $icon_fid = $form_state->getValue(['icon_' . $i, 0]);
            if ($icon_fid) {
                $this->configuration['icon_' . $i] = $icon_fid;
                $file = File::load($icon_fid);
                if ($file) {
                    $file->setPermanent();
                    $file->save();
                }
            }

          // Save formatted texts
            $text_value = $form_state->getValue(['text_' . $i]);
            $this->configuration['text_' . $i] = [
            'value' => $text_value['value'],
            'format' => $text_value['format'],
            ];
        }
    }

  /**
   * {@inheritdoc}
   */
    public function build(): array
    {
        $build = [
        '#theme' => 'interactive_globe',
        '#globe_background' => '',
        '#icons' => [],
        '#texts' => [],
        '#attached' => [
        'library' => [
          'interactive_globe/interactive-globe',
        ],
        ],
        ];

      // Load the background image if configured
        if (!empty($this->configuration['globe_background'])) {
            $file = File::load($this->configuration['globe_background']);
            if ($file) {
                $build['#globe_background'] = $file->createFileUrl();
            }
        }

      // Load icons and configured texts
        for ($i = 1; $i <= 6; $i++) {
            $icon_id = $this->configuration["icon_$i"] ?? null;
            if ($icon_id) {
                $file = File::load($icon_id);
                if ($file) {
                    $build['#icons'][] = $file->createFileUrl();
                } else {
                    $build['#icons'][] = ''; // Handle the case where the file is not found
                }
            } else {
                $build['#icons'][] = ''; // Handle the case where no file ID is configured
            }

          // Ensure formatted text is processed correctly
            if (isset($this->configuration['text_' . $i])) {
                $build['#texts'][] = [
                '#type' => 'processed_text',
                '#text' => $this->configuration['text_' . $i]['value'],
                '#format' => $this->configuration['text_' . $i]['format'],
                '#filter_types_to_skip' => [], // Add or remove filters as needed
                ];
            } else {
              // Provide a fallback if no text is configured
                $build['#texts'][] = [
                '#type' => 'processed_text',
                '#text' => '',
                '#format' => 'restricted_html', // Use the least permissive default text format
                ];
            }
        }

        return $build;
    }
}
